@@ -2,7 +2,7 @@
 from __future__ import annotations
 import logging
 import threading
-from typing import Iterable, TYPE_CHECKING
+from typing import Iterable, TYPE_CHECKING, Union
 from collections import deque
 from enum import IntEnum, auto
 import time
@@ -16,7 +16,7 @@ _LOGGER = logging.getLogger(__name__)
 
 
 class Controller:
-    """Emulated controller input."""
+    """Controller Interface. Sends user input to Remote Play Session."""
 
     class ButtonAction(IntEnum):
         """Button Action Types."""
@@ -28,8 +28,6 @@ class Controller:
     MAX_EVENTS = 16
     STATE_INTERVAL_MAX_MS = 0.200
     STATE_INTERVAL_MIN_MS = 0.100
-    STICK_STATE_MAX = 0x7FFF
-    STICK_STATE_MIN = -0x7FFF
 
     @staticmethod
     def buttons() -> list:
@@ -56,7 +54,7 @@ class Controller:
     def __worker(self):
         """Worker for sending feedback packets. Run in thread."""
         self._should_send.acquire(timeout=1)
-        while not self._session.is_stopped and not self._stop_event.is_set():
+        while self.running:
             self._should_send.acquire(timeout=Controller.STATE_INTERVAL_MAX_MS)
             self.update_sticks()
         self._session = None
@@ -103,8 +101,8 @@ class Controller:
         Will be called automatically if controller has been started.
         """
         if (
-            self._session is None
-            or self._session.is_stopped
+            self.session is None
+            or self.session.is_stopped
             or self.stick_state == self._last_state
         ):
             return
@@ -117,6 +115,8 @@ class Controller:
 
     def _send_event(self):
         """Send controller button event."""
+        if self.session is None or self.session.is_stopped:
+            return
         data = b"".join(self._event_buf)
         if not data:
             return
@@ -134,36 +134,48 @@ class Controller:
         event.pack(buf)
         self._event_buf.appendleft(buf)
 
-    def button(self, name: str, action="tap", delay=0.1):
+    def button(
+        self,
+        name: Union[str, FeedbackEvent.Type],
+        action: Union[str, ButtonAction] = "tap",
+        delay=0.1,
+    ):
         """Emulate pressing or releasing button.
 
         If action is `tap` this method will block by delay.
 
         :param name: The name of button. Use buttons() to show valid buttons.
-        :param action: One of press, release, tap
+        :param action: One of `press`, `release`, `tap`, or `Controller.ButtonAction`.
         :param delay: Delay between press and release. Only used when action is `tap`.
         """
-        try:
-            _action = self.ButtonAction[action.upper()]
-        except KeyError:
-            _LOGGER.error("Invalid Action: %s", action)
-            return
-        try:
-            button = int(FeedbackEvent.Type[name.upper()])
-        except KeyError:
-            _LOGGER.error("Invalid button: %s", name)
+        if isinstance(action, self.ButtonAction):
+            _action = action
         else:
-            if _action == self.ButtonAction.PRESS:
-                self._add_event_buffer(FeedbackEvent(button, is_active=True))
-            elif _action == self.ButtonAction.RELEASE:
-                self._add_event_buffer(FeedbackEvent(button, is_active=False))
-            elif _action == self.ButtonAction.TAP:
-                self._add_event_buffer(FeedbackEvent(button, is_active=True))
-                self._send_event()
-                time.sleep(delay)
-                self.button(name, "release")
+            try:
+                _action = self.ButtonAction[action.upper()]
+            except KeyError:
+                _LOGGER.error("Invalid Action: %s", action)
                 return
+        if isinstance(name, FeedbackEvent.Type):
+            button = int(name)
+        else:
+            try:
+                button = int(FeedbackEvent.Type[name.upper()])
+            except KeyError:
+                _LOGGER.error("Invalid button: %s", name)
+                return
+
+        if _action == self.ButtonAction.PRESS:
+            self._add_event_buffer(FeedbackEvent(button, is_active=True))
+        elif _action == self.ButtonAction.RELEASE:
+            self._add_event_buffer(FeedbackEvent(button, is_active=False))
+        elif _action == self.ButtonAction.TAP:
+            self._add_event_buffer(FeedbackEvent(button, is_active=True))
             self._send_event()
+            time.sleep(delay)
+            self.button(name, self.ButtonAction.RELEASE)
+            return
+        self._send_event()
 
     def stick(
         self,
@@ -190,7 +202,7 @@ class Controller:
         :param stick_name: The stick to move. One of 'left' or 'right'
         :param axis: The axis to move. One of 'x' or 'y'
         :param value: The value to move stick to. Must be between -1.0 and 1.0
-        :param point: An interable of two floats, which represent coordinates.
+        :param point: An iterable of two floats, which represent coordinates.
             Point takes precedence over axis and value.
             The first value represents the x axis and the second represents the y axis
         """
@@ -237,3 +249,15 @@ class Controller:
     def stick_state(self) -> ControllerState:
         """Return stick state."""
         return self._stick_state
+
+    @property
+    def running(self) -> bool:
+        """Return True if running."""
+        if not self._session:
+            return False
+        return not self._session.is_stopped and not self._stop_event.is_set()
+
+    @property
+    def session(self) -> Session:
+        """Return Session."""
+        return self._session
