@@ -169,7 +169,7 @@ class AVReceiver(abc.ABC):
 
     def get_video_codec(self):
         """Get Codec Context."""
-        codec_name = self._session.video_format
+        codec_name = self._session.codec
         self.video_decoder = AVReceiver.video_codec(codec_name)
         try:
             self.video_decoder.open()
@@ -186,14 +186,6 @@ class AVReceiver(abc.ABC):
         """Set Session."""
         self._session = session
 
-    def notify_started(self):
-        """Notify session that receiver has started."""
-        self._session.receiver_started.set()
-
-    def start(self):
-        """Start receiver."""
-        self.notify_started()
-
     def decode_video_frame(self, buf: bytes) -> av.VideoFrame:
         """Return decoded Video Frame."""
         if not self.video_decoder:
